# $Id: shared.sh.in,v 1.35 2002/08/15 15:58:20 cph Exp $
#
# Copyright (c) 2000-2002 Massachusetts Institute of Technology
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

# Shared code for network-interface control scripts.

STATE_DIR="/var/state/laptop-net"
STATE_FILE="${STATE_DIR}/state"
TARGET_FILE="${STATE_DIR}/target"
SCHEME_FILE="${STATE_DIR}/scheme"
SCHEME_TEMP="${SCHEME_FILE}.tmp"
SCHEME_SOURCE_FILE="${SCHEME_FILE}-source"
RESUME_FILE="${STATE_DIR}/resume"
PROFILE_FILE="${STATE_DIR}/profile"

LOCK_FILE="/var/lock/laptop-net/lock"
NOW=`date +%s`

# Don't do anything if the state files are missing.
[ -f "${STATE_FILE}" ] || exit 0
[ -f "${TARGET_FILE}" ] || exit 0
[ -f "${SCHEME_FILE}" ] || exit 0

IP_MAP_FILE="/etc/laptop-net/ip-map"
SCHEME_CONFIG_FILE="/etc/laptop-net/schemes"
CONFIG_FILE="/etc/laptop-net/config"
RESOLV_CONF="/etc/resolv.conf"
SCHEME_DIR="/var/lib/laptop-net/schemes"
PROFILE_CHANGE="/usr/share/laptop-net/profile-change"
ARP_DISCOVERY="/usr/libexec/laptop-net/arp-discovery"

LOGGER_PROGRAM="/usr/bin/logger"
if [ -x "${LOGGER_PROGRAM}" ]; then
    LOGGER="${LOGGER_PROGRAM} -p daemon.info -t laptop-net"
else
    LOGGER="true"
fi

LINKUP="/usr/libexec/laptop-net/linkup"
MKTONE="/usr/libexec/laptop-net/mktone 150"
MKTONE_UP=1000
MKTONE_DOWN=2000
MKTONE_ERROR=4000

INTERFACE="eth0"

kernel_at_least_2_4_p ()
{
    MAJOR=$(uname --release | awk -F. '{ print $1 }')
    MINOR=$(uname --release | awk -F. '{ print $2 }')
    if [ ${MAJOR} -lt 2 ]; then
	return 1
    elif [ ${MAJOR} -eq 2 ] && [ ${MINOR} -lt 4 ]; then
	return 1
    else
	return 0
    fi
}

MODULE_NAME=
MII_SUPPORTED=
. "${CONFIG_FILE}"

if [ "${MODULE_NAME}" = "none" ]; then
    MODULE_NAME=
fi

if [ -z "${MII_SUPPORTED}" -a -n "${MODULE_NAME}" ]; then
    if kernel_at_least_2_4_p; then
	case "${MODULE_NAME}" in
	3c59x | 8139too | au1000_eth | e100 | eepro100 | epic100 | fealnx | hamachi \
	| ioc3-eth | natsemi | pcnet32 | sis900 | starfire | sundance \
	| sungem | tlan | tulip | via-rhine | winbond-840 | yellowfin)
	    MII_SUPPORTED="yes"
	    ;;
	*)
	    MII_SUPPORTED="no"
	    ;;
	esac
    else
	# 2.2.x kernels
	case "${MODULE_NAME}" in
	3c59x | e100 | eepro100 | epic100 | old_tulip | pcnet32 | rtl8139 | sis900 \
	| tulip | via-rhine | yellowfin)
	    MII_SUPPORTED="yes"
	    ;;
	*)
	    MII_SUPPORTED="no"
	    ;;
	esac
    fi
fi

[ -n "${MII_SUPPORTED}" ] || MII_SUPPORTED="yes"

# Disable use of LINKUP if MII not supported.
[ "${MII_SUPPORTED}" = "yes" ] || LINKUP="/bin/true"

bringup ()
{
    why="${1}"
    if [ "$(cat "${TARGET_FILE}")" != "up" ]; then
	return 0
    fi
    lock_state || return 1
    choose_scheme
    /sbin/ip addr flush dev ${INTERFACE}
    IFCFG_FILE="${SCHEME_DIR}/${SCHEME}"
    if write_ifcfg_file "${IFCFG_FILE}"; then
	${LOGGER} "Starting built-in network interface"
	ln -sf "${IFCFG_FILE}" "/etc/sysconfig/network-scripts/ifcfg-${INTERFACE}"
	ifup "${INTERFACE}"
	write_resolv_conf
	"${PROFILE_CHANGE}" "${INTERFACE}"
	if "${LINKUP}" "${INTERFACE}"; then
	    ${MKTONE} "${MKTONE_UP}"
	else
	    ${MKTONE} "${MKTONE_ERROR}"
	fi
    fi
    unlock_state
}

bringdown ()
{
    why="${1}"
    if [ "$(cat "${STATE_FILE}")" = "down" ]; then
	return 0
    fi
    lock_state || return 1
    read_scheme
    IFCFG_FILE="${SCHEME_DIR}/${SCHEME}"
    if write_ifcfg_file "${IFCFG_FILE}"; then
	# Don't try this if the interface is already down.
	if "${LINKUP}" "${INTERFACE}"; then
	    shut_down_nfs
	fi
	${LOGGER} "Stopping built-in network interface"
	ifdown "${INTERFACE}"
	rm "/etc/sysconfig/network-scripts/ifcfg-${INTERFACE}" || true
	"${PROFILE_CHANGE}" "${INTERFACE}"
	${MKTONE} "${MKTONE_DOWN}"
    fi
    unlock_state
}

lock_state ()
{
    retry=10
    timeout=60
    while [ $retry -gt 0 ]; do
        mv $LOCK_FILE $LOCK_FILE.$NOW && return 0
        sleep 1
        retry=`expr $retry - 1`
    done
    for old_lock in $LOCK_FILE.*; do
        sec=`expr $old_lock : $LOCK_FILE.\([0-9][0-9]*\)$`
        if [ -n "$sec" ] && [ `expr $NOW - $sec` -gt $timeout ]; then
            mv $old_lock $LOCK_FILE.$NOW && return 0
        fi
     done
    ${LOGGER} "Unable to lock \"${STATE_FILE}\"."
    return 1
}

unlock_state ()
{
    mv $LOCK_FILE.$NOW $LOCK_FILE
}

choose_scheme ()
{
    rm -f "${SCHEME_TEMP}"
    if "${ARP_DISCOVERY}" "${INTERFACE}" "${IP_MAP_FILE}" > "${SCHEME_TEMP}"
    then
	NEW_SCHEME="$(cat "${SCHEME_TEMP}")"
	[ "${NEW_SCHEME}" ] || NEW_SCHEME="@NO-CHOICES@"
    else
	NEW_SCHEME="@NO-CHOICES@"
    fi
    case "${NEW_SCHEME}" in
    "@NO-CHOICES@")
	${LOGGER} "No change to network scheme"
	;;
    "@NO-RESPONSES@")
	if [ "$(read_scheme_source)" = "arp-discovery" ]; then
	    write_scheme "default" "arp-discovery"
	else
	    ${LOGGER} "No change to network scheme"
	fi
	;;
    *)
	write_scheme "${NEW_SCHEME}" "arp-discovery"
	;;
    esac
    rm -f "${SCHEME_TEMP}"
    read_scheme
}

read_scheme ()
{
    SCHEME="$(cat "${SCHEME_FILE}")"
    DHCP=
    ADDRESS=
    NETMASK=
    BROADCAST=
    NETWORK=
    GATEWAY=
    DOMAIN=
    SEARCH=
    NAMESERVERS=
    START_CMD=
    STOP_CMD=
    if [ -e "${SCHEME_CONFIG_FILE}" ]; then
	. "${SCHEME_CONFIG_FILE}"
    else
	[ "${SCHEME}" = "offline" ] || DHCP="yes"
    fi
}

write_scheme ()
{
    ${LOGGER} "Network scheme \"${1}\" selected by \"${2}\"."
    echo "${1}" > "${SCHEME_FILE}"
    echo "${2}" > "${SCHEME_SOURCE_FILE}"
}

read_scheme_source ()
{
    if [ -r "${SCHEME_SOURCE_FILE}" ]; then
	cat "${SCHEME_SOURCE_FILE}"
    else
	echo "unknown"
    fi
}

shut_down_nfs ()
{
    # Shut down all NFS mounts on this interface.
    # (Shamelessly appropriated from pcmcia-cs.)
    echo -n "Unmounting remote filesystems"
    nfsstop ()
    {
	local HOST MT
	if read HOST MT ; then
	    nfsstop
	    case "${HOST}" in
	    localhost | 127.0.0.1)
		;;
	    *)
		if ifuser "${INTERFACE}" "${HOST}" ; then
		    fuser -k -m "${MT}" > /dev/null
		    umount -f -v "${MT}"
		fi
		;;
	    esac
	fi
    }
    mount -t nfs | sed -e 's/:.* on \(.*\) type .*/ \1/' | nfsstop
    echo "."
}

load_module ()
{
    if [ -n "${MODULE_NAME}" ]; then
	modprobe "${MODULE_NAME}" || true
    fi
    [ "${MII_SUPPORTED}" = "yes" ] || bringup connect
    return 0
}

unload_module ()
{
    if [ -n "${MODULE_NAME}" ]; then
	modprobe -r "${MODULE_NAME}" || true
    fi
    return 0
}

kernel_supports_power_management ()
{
    # For now, don't take advantage of 2.4.x suspend support.
    return 1
}

write_ifcfg_file ()
{
    FILENAME="${1}"
    if [ ! -e "${FILENAME}" ] || [ "${SCHEME_CONFIG_FILE}" -nt "${FILENAME}" ]
    then
	rm -f "${FILENAME}"
	echo "DEVICE=${INTERFACE}" > "${FILENAME}"
	echo "ONBOOT=no" >> "${FILENAME}"

	if [ "${DHCP}" = "yes" ]; then
	    echo "BOOTPROTO=dhcp" >> "${FILENAME}"
	    WRITE_FILE="yes"
	elif [ -n "${ADDRESS}" ] && [ -n "${NETMASK}" ]; then
	    echo "IPADDR=${ADDRESS}" >> "${FILENAME}"
	    echo "NETMASK=${NETMASK}" >> "${FILENAME}"
	    if [ -n "${BROADCAST}" ]; then
		echo "BROADCAST=${BROADCAST}" >> "${FILENAME}"
	    fi
	    if [ -n "${NETWORK}" ]; then
		echo "NETWORK=${NETWORK}" >> "${FILENAME}"
	    fi
	    if [ -n "${GATEWAY}" ]; then
		echo "GATEWAY=${GATEWAY}" >> "${FILENAME}"
	    fi
	    WRITE_FILE="yes"
	else
	    WRITE_FILE="no"
	fi

	if [ "${WRITE_FILE}" = "yes" ]; then
	    if [ -n "${START_CMD}" ]; then
		echo "    up ${START_CMD}" >> "${FILENAME}"
	    fi
	    if [ -n "${STOP_CMD}" ]; then
		echo "    down ${STOP_CMD}" >> "${FILENAME}"
	    fi
	    chmod 644 "${FILENAME}"
	fi
    else
	WRITE_FILE="yes"
    fi

    [ "${WRITE_FILE}" = "yes" ]
}

write_resolv_conf ()
{
    if [ -n "${RESOLV_CONF}" ]; then
	RESOLV_TMP="${RESOLV_CONF}.LTN"
	RESOLV_BAK="${RESOLV_CONF}.old"
	# Delete previous entries for this interface.
	SED_SCRIPT="/# ${INTERFACE} begin/,/# ${INTERFACE} end/d"
	# Delete other related entries.
	[ -n "${SEARCH}" ] && SED_SCRIPT="${SED_SCRIPT}; /^search/d"
	[ -n "${NAMESERVERS}" ] && SED_SCRIPT="${SED_SCRIPT}; /^nameserver/d"
	[ -n "${DOMAIN}" ] && SED_SCRIPT="${SED_SCRIPT}; /^domain/d"
	# Delete previously commented-out entries.
	SED_SCRIPT="${SED_SCRIPT}; /^#search/d"
	SED_SCRIPT="${SED_SCRIPT}; /^#nameserver/d"
	SED_SCRIPT="${SED_SCRIPT}; /^#domain/d"
	# Create the new resolv.conf file.
	if [ -n "${DOMAIN}${SEARCH}${NAMESERVERS}" ]; then
	    echo "# ${INTERFACE} begin" > "${RESOLV_TMP}"
	    [ -z "${DOMAIN}" ] || echo "domain ${DOMAIN}" >> "${RESOLV_TMP}"
	    [ -z "${SEARCH}" ] || echo "search ${SEARCH}" >> "${RESOLV_TMP}"
	    for NAMESERVER in ${NAMESERVERS}; do
		echo "nameserver ${NAMESERVER}" >> "${RESOLV_TMP}"
	    done
	    echo "# ${INTERFACE} end" >> "${RESOLV_TMP}"
	    sed -e "${SED_SCRIPT}" "${RESOLV_CONF}" >> "${RESOLV_TMP}"
	else
	    sed -e "${SED_SCRIPT}" "${RESOLV_CONF}" > "${RESOLV_TMP}"
	fi
	if [ -e "${RESOLV_CONF}" ]; then
	    # Do nothing if no changes. 
	    if cmp --quiet "${RESOLV_TMP}" "${RESOLV_CONF}"; then
		rm -f "${RESOLV_TMP}"
		return 0
	    fi
	    # Back up current file.
	    rm -f "${RESOLV_BAK}"
	    ln -f "${RESOLV_CONF}" "${RESOLV_BAK}"
	fi
	# Install new file.
	chmod 644 "${RESOLV_TMP}"
	mv -f "${RESOLV_TMP}" "${RESOLV_CONF}"
    fi
}
